@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -21,6 +22,7 @@
 #ifndef MEDIASTREAM_H
 #define MEDIASTREAM_H
 
+#include "ortp/port.h"
 #include <ortp/ortp.h>
 #include <ortp/event.h>
 #include <ortp/nack.h>
@@ -94,7 +96,7 @@ typedef struct _MediaStream MediaStream;
 typedef void (*media_stream_process_rtcp_callback_t)(MediaStream *stream, mblk_t *m);
 
 struct _MSMediaStreamSessions{
-	RtpSession *rtp_session;
+    RtpSession *rtp_session;
 	MSSrtpCtx* srtp_context;
 	MSZrtpContext *zrtp_context;
 	MSDtlsSrtpContext *dtls_context;
@@ -119,7 +121,7 @@ typedef enum _MSStreamState{
 typedef enum MediaStreamDir{
 	MediaStreamSendRecv,
 	MediaStreamSendOnly,
-	MediaStreamRecvOnly
+	MediaStreamRecvOnly,
 }MediaStreamDir;
 
 /**
@@ -147,7 +149,7 @@ struct _MediaStream {
 	bool_t rc_enable;
 	bool_t is_beginning;
 	bool_t owns_sessions;
-	bool_t pad;
+	bool_t stun_allowed; /* Whether sending basic stun packet as keepalive is permitted, default is yes */
 	/**
 	 * defines encoder target network bit rate, uses #media_stream_set_target_network_bitrate() setter.
 	 * */
@@ -193,6 +195,8 @@ MS2_PUBLIC void media_stream_enable_adaptive_jittcomp(MediaStream *stream, bool_
 
 MS2_PUBLIC void media_stream_set_ice_check_list(MediaStream *stream, IceCheckList *cl);
 
+MS2_PUBLIC void media_stream_set_stun_allowed(MediaStream *stream, bool_t value);
+
 /*
  * deprecated, use media_stream_set_srtp_recv_key and media_stream_set_srtp_send_key.
 **/
@@ -303,8 +307,6 @@ MS2_PUBLIC void media_stream_iterate(MediaStream * stream);
 
 MS2_PUBLIC void media_stream_set_direction(MediaStream *stream, MediaStreamDir dir);
 
-MS2_PUBLIC void media_stream_reset_zrtp_context(MediaStream *stream);
-
 MS2_PUBLIC MediaStreamDir media_stream_get_direction(const MediaStream *stream);
 
 /**
@@ -319,6 +321,22 @@ MS2_PUBLIC MSStreamState media_stream_get_state(const MediaStream *stream);
 
 MS2_PUBLIC OrtpEvDispatcher* media_stream_get_event_dispatcher(const MediaStream *stream);
 
+/**
+ * Retrieve the receive ssrc of the stream
+ *
+ * @param stream the media stream
+ * @return the receive ssrc of the stream
+ */
+MS2_PUBLIC uint32_t media_stream_get_recv_ssrc(const MediaStream *stream);
+
+/**
+ * Retrieve the send ssrc of the stream
+ *
+ * @param stream the media stream
+ * @return the send ssrc of the stream
+ */
+MS2_PUBLIC uint32_t media_stream_get_send_ssrc(const MediaStream *stream);
+
 typedef enum EchoLimiterType{
 	ELInactive,
 	ELControlMic,
@@ -339,7 +357,8 @@ typedef enum MSResourceType{
 	MSResourceRtp,
 	MSResourceCamera,
 	MSResourceSoundcard,
-	MSResourceVoid
+	MSResourceVoid,
+	MSResourceItc
 }MSResourceType;
 
 MS2_PUBLIC const char *ms_resource_type_to_string(MSResourceType type);
@@ -349,6 +368,7 @@ MS2_PUBLIC const char *ms_resource_type_to_string(MSResourceType type);
  * type must be set to one the member of the MSResourceType enum, and the correspoding
  * resource argument must be set: the file name (const char*) for MSResourceFile,
  * the RtpSession for MSResourceRtp, an MSWebCam for MSResourceCamera, an MSSndCard for MSResourceSoundcard.
+ * an MSFilter for MSResourceItc
  * @warning due to implementation, if RTP is to be used for input and output, the same RtpSession must be passed for both sides.
  */
 typedef struct _MSMediaResource{
@@ -359,6 +379,7 @@ typedef struct _MSMediaResource{
 		RtpSession *session;
 		MSWebCam *camera;
 		MSSndCard *soundcard;
+		MSFilter *itc;
 	};
 }MSMediaResource;
 
@@ -381,6 +402,9 @@ typedef struct _MSMediaStreamIO {
 
 MS2_PUBLIC bool_t ms_media_stream_io_is_consistent(const MSMediaStreamIO *io);
 
+typedef void (*AudioStreamIsSpeakingCallback)(void *user_pointer, uint32_t speaker_ssrc, bool_t is_speaking);
+typedef void (*AudioStreamIsMutedCallback)(void *user_pointer, uint32_t speaker_ssrc, bool_t is_muted);
+typedef void (*MSAudioRouteChangedCallback)(void* audioStream, bool_t needReloadSoundDevices, char* newInputPort, char* newOutputPort);
 struct _AudioStream
 {
 	MediaStream ms;
@@ -435,6 +459,7 @@ struct _AudioStream
 	MSAudioRoute audio_route;
 	bool_t play_dtmfs;
 	bool_t use_ec;
+	bool_t force_software_ec;
 	bool_t use_gc;
 	bool_t use_agc;
 
@@ -444,6 +469,16 @@ struct _AudioStream
 	bool_t is_ec_delay_set;
 	bool_t disable_record_on_mute;
 	float last_mic_gain_level_db;
+	struct _AudioStreamVolumes *participants_volumes;
+	int mixer_to_client_extension_id;
+	int client_to_mixer_extension_id;
+	uint32_t speaking_ssrc;
+	AudioStreamIsSpeakingCallback is_speaking_cb;
+	AudioStreamIsMutedCallback is_muted_cb;
+	void *user_pointer;
+	bool_t is_speaking;
+	MSAudioRouteChangedCallback audio_route_changed_cb;
+	void * audio_route_changed_cb_user_data;
 };
 
 /**
@@ -451,7 +486,6 @@ struct _AudioStream
 **/
 typedef struct _AudioStream AudioStream;
 
-
 /* start a thread that does sampling->encoding->rtp_sending|rtp_receiving->decoding->playing */
 MS2_PUBLIC AudioStream *audio_stream_start(MSFactory* factory, RtpProfile * prof, int locport, const char *remip,
 				 int remport, int payload_type, int jitt_comp, bool_t echo_cancel);
@@ -624,11 +658,16 @@ MS2_PUBLIC void audio_stream_enable_automatic_gain_control(AudioStream *stream,
  *  */
 MS2_PUBLIC void audio_stream_set_echo_canceller_params(AudioStream *st, int tail_len_ms, int delay_ms, int framesize);
 
-
 /**
  * to be done before start
  *  */
 MS2_PUBLIC void audio_stream_enable_echo_canceller(AudioStream *st, bool_t enabled);
+
+/**
+ * Forces the use of a software echo canceller if available instead of hardwared/built-in one.
+*/
+MS2_PUBLIC void audio_stream_force_software_echo_canceller(AudioStream *st, bool_t force);
+
 /**
  * enable adaptive rate control
  * */
@@ -773,9 +812,14 @@ MS2_PUBLIC void audio_stream_stop (AudioStream * stream);
  *  */
 MS2_PUBLIC int audio_stream_send_dtmf (AudioStream * stream, char dtmf);
 
+/**
+ *  Are telephone events supported? Used to determine if DTMFs can be sent out-of-band
+ */
+MS2_PUBLIC bool_t audio_stream_supports_telephone_events(AudioStream *stream);
+
 MS2_PUBLIC MSFilter *audio_stream_get_local_player(AudioStream *stream);
 
-MS2_PUBLIC int audio_stream_mixed_record_open(AudioStream *st, const char*filename);
+MS2_PUBLIC int audio_stream_set_mixed_record_file(AudioStream *st, const char*filename);
 
 MS2_PUBLIC int audio_stream_mixed_record_start(AudioStream *st);
 
@@ -835,8 +879,75 @@ static MS2_INLINE RtpSession * audio_stream_get_rtp_session(const AudioStream *s
 	return media_stream_get_rtp_session(&stream->ms);
 }
 
+/**
+ * Sets the header extension id for mixer to client audio level indication.
+ * This has to be called before starting the audio stream.
+ *
+ * @param stream the audio stream
+ * @param extension_id the extension id
+ */
+MS2_PUBLIC void audio_stream_set_mixer_to_client_extension_id(AudioStream *stream, int extension_id);
+
+/**
+ * Sets the header extension id for client to mixer audio level indication.
+ * This has to be called before starting the audio stream.
+ *
+ * @param stream the audio stream
+ * @param extension_id the extension id
+ */
+MS2_PUBLIC void audio_stream_set_client_to_mixer_extension_id(AudioStream *stream, int extension_id);
+
+MS2_PUBLIC void audio_stream_set_is_speaking_callback(AudioStream *s, AudioStreamIsSpeakingCallback cb, void *user_pointer);
+MS2_PUBLIC void audio_stream_set_is_muted_callback(AudioStream *s, AudioStreamIsMutedCallback cb, void *user_pointer);
+
+/**
+ * Retrieve the volume of the given participant.
+ *
+ * @param stream the audio stream
+ * @param participant_ssrc the ssrc of the participant
+ * @return the volume of the participant in dbm0, if participant isn't found it will return the lowest volume.
+ */
+MS2_PUBLIC int audio_stream_get_participant_volume(const AudioStream *stream, uint32_t participant_ssrc);
+
+/**
+ * Retrieve the receive ssrc of the stream
+ *
+ * @param stream the audio stream
+ * @return the receive ssrc of the stream
+ */
+MS2_PUBLIC uint32_t audio_stream_get_recv_ssrc(const AudioStream *stream);
+
+/**
+ * Retrieve the send ssrc of the stream
+ *
+ * @param stream the audio stream
+ * @return the send ssrc of the stream
+ */
+MS2_PUBLIC uint32_t audio_stream_get_send_ssrc(const AudioStream *stream);
 
 
+/* Map api to be able to keep participants volumes */
+typedef struct _AudioStreamVolumes AudioStreamVolumes;
+
+#define AUDIOSTREAMVOLUMES_NOT_FOUND INT16_MIN
+
+MS2_PUBLIC AudioStreamVolumes* audio_stream_volumes_new(void);
+MS2_PUBLIC void audio_stream_volumes_delete(AudioStreamVolumes *volumes);
+
+MS2_PUBLIC void audio_stream_volumes_insert(AudioStreamVolumes *volumes, uint32_t ssrc, int volume);
+MS2_PUBLIC void audio_stream_volumes_erase(AudioStreamVolumes *volumes, uint32_t ssrc);
+MS2_PUBLIC void audio_stream_volumes_clear(AudioStreamVolumes *volumes);
+
+MS2_PUBLIC int audio_stream_volumes_size(AudioStreamVolumes *volumes);
+MS2_PUBLIC int audio_stream_volumes_find(AudioStreamVolumes *volumes, uint32_t ssrc);
+MS2_PUBLIC int audio_stream_volumes_append(AudioStreamVolumes *volumes, AudioStreamVolumes *append);
+
+MS2_PUBLIC void audio_stream_volumes_reset_values(AudioStreamVolumes *volumes);
+
+MS2_PUBLIC void audio_stream_volumes_populate_audio_levels(AudioStreamVolumes *volumes, rtp_audio_level_t *audio_levels);
+
+MS2_PUBLIC bool_t audio_stream_volumes_is_speaking(AudioStreamVolumes *volumes);
+MS2_PUBLIC uint32_t audio_stream_volumes_get_best(AudioStreamVolumes *volumes);
 /**
  * @}
 **/
@@ -851,6 +962,7 @@ typedef void (*VideoStreamRenderCallback)(void *user_pointer, const MSPicture *l
 typedef void (*VideoStreamEventCallback)(void *user_pointer, const MSFilter *f, const unsigned int event_id, const void *args);
 typedef void (*VideoStreamCameraNotWorkingCallback)(void *user_pointer, const MSWebCam *old_webcam);
 typedef void (*VideoStreamEncoderControlCb)(struct _VideoStream *, unsigned int method_id, void *arg, void *user_data);
+typedef void (*VideoStreamCsrcChangedCb)(void *user_pointer, uint32_t new_csrc);
 
 struct _MediastreamVideoStat
 {
@@ -862,6 +974,13 @@ struct _MediastreamVideoStat
 
 typedef struct _MediastreamVideoStat MediaStreamVideoStat;
 
+typedef enum _MSVideoContent{
+	MSVideoContentDefault,
+	MSVideoContentSpeaker,
+	MSVideoContentThumbnail
+} MSVideoContent;
+
+
 struct _VideoStream
 {
 	MediaStream ms;
@@ -878,8 +997,11 @@ struct _VideoStream
 	MSFilter *tee2;
 	MSFilter *tee3;
 	MSFilter *void_source;
+	MSFilter *itcsink;
+	MSFilter *forward_sink;
 	MSVideoSize sent_vsize;
 	MSVideoSize preview_vsize;
+	MSVideoSize max_sent_vsize;
 	float forced_fps; /*the target fps explicitely set by application, overrides internally selected fps*/
 	float configured_fps; /*the fps that was configured to the encoder. It might be different from the one really obtained from camera.*/
 	float real_fps; /*the fps obtained from camera.*/
@@ -909,15 +1031,23 @@ struct _VideoStream
 	MediaStreamVideoStat ms_video_stat;
 	VideoStreamEncoderControlCb encoder_control_cb;
 	void *encoder_control_cb_user_data;
+	MSVideoDisplayMode display_mode;
+	int frame_marking_extension_id;
+	char *label;
+	MSVideoContent content;
 	bool_t use_preview_window;
 	bool_t enable_qrcode_decoder;
 	bool_t freeze_on_error;
 	bool_t display_filter_auto_rotate_enabled;
-	
 	bool_t source_performs_encoding;
 	bool_t output_performs_decoding;
 	bool_t player_active;
 	bool_t staticimage_webcam_fps_optimization; /* if TRUE, the StaticImage webcam will ignore the fps target in order to save CPU time. Default is TRUE */
+	bool_t is_forwarding;
+	bool_t wait_for_frame_decoded;
+	VideoStreamCsrcChangedCb csrc_changed_cb;
+	void *csrc_changed_cb_user_data;
+	uint32_t new_csrc;
 };
 
 typedef struct _VideoStream VideoStream;
@@ -950,6 +1080,10 @@ MS2_PUBLIC void video_stream_set_render_callback(VideoStream *s, VideoStreamRend
 MS2_PUBLIC void video_stream_set_event_callback(VideoStream *s, VideoStreamEventCallback cb, void *user_pointer);
 MS2_PUBLIC void video_stream_set_camera_not_working_callback(VideoStream *s, VideoStreamCameraNotWorkingCallback cb, void *user_pointer);
 MS2_PUBLIC void video_stream_set_display_filter_name(VideoStream *s, const char *fname);
+MS2_PUBLIC void video_stream_set_label(VideoStream *s, const char *label);
+MS2_PUBLIC void video_stream_set_content(VideoStream *s, MSVideoContent content);
+MS2_PUBLIC MSVideoContent video_stream_get_content(const VideoStream *vs);
+
 MS2_PUBLIC int video_stream_start_with_source(VideoStream *stream, RtpProfile *profile, const char *rem_rtp_ip, int rem_rtp_port,
 		const char *rem_rtcp_ip, int rem_rtcp_port, int payload, int jitt_comp, MSWebCam* cam, MSFilter* source);
 MS2_PUBLIC int video_stream_start(VideoStream * stream, RtpProfile *profile, const char *rem_rtp_ip, int rem_rtp_port, const char *rem_rtcp_ip,
@@ -972,6 +1106,13 @@ MS2_PUBLIC int video_stream_start_with_files(VideoStream *stream, RtpProfile *pr
 MS2_PUBLIC int video_stream_start_from_io(VideoStream *stream, RtpProfile *profile, const char *rem_rtp_ip, int rem_rtp_port,
 	const char *rem_rtcp_ip, int rem_rtcp_port, int payload_type, const MSMediaStreamIO *io);
 
+/**
+ * Link a video stream with ItcSink filter. Used for starting another video stream.
+ *
+ * @param[in] stream VideoStream object previously created with video_stream_new().
+ */
+MS2_PUBLIC void link_video_stream_with_itc_sink(VideoStream *stream);
+
 MS2_PUBLIC void video_stream_prepare_video(VideoStream *stream);
 MS2_PUBLIC void video_stream_unprepare_video(VideoStream *stream);
 
@@ -992,6 +1133,7 @@ MS2_PUBLIC const MSWebCam * video_stream_get_camera(const VideoStream *stream);
  */
 MS2_PUBLIC MSFilter* video_stream_get_source_filter(const VideoStream* stream);
 
+MS2_PUBLIC void video_preview_stream_change_camera(VideoStream* stream, MSWebCam* cam);
 MS2_PUBLIC void video_stream_change_camera(VideoStream *stream, MSWebCam *cam);
 
 MS2_PUBLIC void video_stream_change_camera_skip_bitrate(VideoStream *stream, MSWebCam *cam);
@@ -1032,6 +1174,17 @@ MS2_PUBLIC void video_stream_change_camera_skip_bitrate(VideoStream *stream, MSW
  */
 MS2_PUBLIC MSFilter* video_stream_change_source_filter(VideoStream *stream, MSWebCam* cam, MSFilter* filter, bool_t keep_previous_source );
 
+/**
+ * @brief This function forwards the video from the source stream to the specifed one.
+ * @details This will connect the received video stream from source and will pass it to current stream
+ * to send. Using \ref video_stream_change_camera will stop the forwarding and use the provided camera.
+ * If the source stream is stopped before this stream, a call to \ref video_stream_change_camera is also
+ * required in order to restart the graph.
+ * @param stream the video stream to modify
+ * @param source the video stream that will be used as a source
+ */
+MS2_PUBLIC void video_stream_forward_source_stream(VideoStream *stream, VideoStream *source);
+
 /**
  * @brief This is the same function as \ref video_stream_change_source_filter() called with keep_source=1, but
  *  the new filter will be created from the MSWebcam that is passed as argument.
@@ -1044,7 +1197,8 @@ MS2_PUBLIC MSFilter* video_stream_change_camera_keep_previous_source(VideoStream
 
 
 /* Calling video_stream_set_sent_video_size() or changing the bitrate value in the used PayloadType during a stream is running does nothing.
-The following function allows to take into account new parameters by redrawing the sending graph*/
+The following functions allow to take into account new parameters by redrawing the sending graph*/
+MS2_PUBLIC void video_preview_stream_update_video_params(VideoStream *stream);
 MS2_PUBLIC void video_stream_update_video_params(VideoStream *stream);
 /*function to call periodically to handle various events */
 MS2_PUBLIC void video_stream_iterate(VideoStream *stream);
@@ -1119,14 +1273,17 @@ MS2_PUBLIC float video_stream_get_sent_framerate(const VideoStream *stream);
 MS2_PUBLIC float video_stream_get_received_framerate(const VideoStream *stream);
 
 MS2_PUBLIC void video_stream_enable_self_view(VideoStream *stream, bool_t val);
+MS2_PUBLIC void * video_stream_create_native_window_id(VideoStream *stream);
 MS2_PUBLIC void * video_stream_get_native_window_id(VideoStream *stream);
 MS2_PUBLIC void video_stream_set_native_window_id(VideoStream *stream, void *id);
+MS2_PUBLIC void * video_stream_create_native_preview_window_id(VideoStream *stream);
 MS2_PUBLIC void video_stream_set_native_preview_window_id(VideoStream *stream, void *id);
 MS2_PUBLIC void * video_stream_get_native_preview_window_id(VideoStream *stream);
 MS2_PUBLIC void video_stream_use_preview_video_window(VideoStream *stream, bool_t yesno);
 MS2_PUBLIC void video_stream_set_device_rotation(VideoStream *stream, int orientation);
 MS2_PUBLIC void video_stream_show_video(VideoStream *stream, bool_t show);
 MS2_PUBLIC void video_stream_set_freeze_on_error(VideoStream *stream, bool_t yesno);
+MS2_PUBLIC void video_stream_set_display_mode(VideoStream *stream, MSVideoDisplayMode mode);
 
 /**
  * @brief Gets the camera sensor rotation.
@@ -1237,6 +1394,15 @@ MS2_PUBLIC void video_stream_use_video_preset(VideoStream *stream, const char *p
  */
 MS2_PUBLIC const char* video_stream_get_video_preset(VideoStream *stream);
 
+/**
+ * Sets the header extension id for frame marking.
+ * This has to be called before starting the video stream.
+ *
+ * @param stream the video stream
+ * @param extension_id the extension id
+ */
+MS2_PUBLIC void video_stream_set_frame_marking_extension_id(VideoStream *stream, int extension_id);
+
 
 /**
  * Open a player to play a video file (mkv) to remote end.
@@ -1257,6 +1423,18 @@ MS2_PUBLIC void video_stream_close_remote_record(VideoStream *stream);
 MS2_PUBLIC void video_stream_enable_retransmission_on_nack(VideoStream *stream, bool_t enable);
 MS2_PUBLIC void video_stream_set_retransmission_on_nack_max_packet(VideoStream *stream, unsigned int max);
 
+MS2_PUBLIC void video_stream_enable_fec(VideoStream *stream, char* local_ip, int local_port, int local_rtcp_port, char* remote_ip, int remote_port, int L, int D);
+
+/**
+ * Sets the max size the sent video stream can reach.
+ *
+ * @param stream the video stream
+ * @param max the max size the sent videostream can reach. 0x0 will remove this limitation.
+ */
+MS2_PUBLIC void video_stream_set_sent_video_size_max(VideoStream *stream, MSVideoSize max);
+
+MS2_PUBLIC void video_stream_set_csrc_changed_callback(VideoStream *stream, VideoStreamCsrcChangedCb cb, void *user_pointer);
+
 /**
  * Small API to display a local preview window.
 **/
@@ -1267,6 +1445,7 @@ MS2_PUBLIC VideoPreview * video_preview_new(MSFactory *factory);
 #define video_preview_set_event_callback(p,c,u) video_stream_set_event_callback(p,c,u)
 #define video_preview_set_size(p,s) video_stream_set_sent_video_size(p,s)
 #define video_preview_set_display_filter_name(p,dt) video_stream_set_display_filter_name(p,dt)
+#define video_preview_create_native_window_id(p) video_stream_create_native_preview_window_id(p)
 #define video_preview_set_native_window_id(p,id) video_stream_set_native_preview_window_id(p,id)
 #define video_preview_get_native_window_id(p) video_stream_get_native_preview_window_id(p)
 #define video_preview_set_fps(p,fps) video_stream_set_fps((VideoStream*)p,fps)
@@ -1311,19 +1490,28 @@ MS2_PUBLIC RtpSession * ms_create_duplex_rtp_session(const char* local_ip, int l
  */
 MS2_PUBLIC void audio_stream_set_audio_route(AudioStream *stream, MSAudioRoute route);
 
+/**
+ *  Set a callback to be called when an audio route change is notified (IOS only)
+ * @param[in] stream The AudioStream object
+ * @param[in] callback The function to call when receiving the route change event
+ * @param[in] audio_route_changed_cb_user_data Context for the callback
+ */
+MS2_PUBLIC void audio_stream_set_audio_route_changed_callback(AudioStream *stream, MSAudioRouteChangedCallback callback, void *audio_route_changed_cb_user_data);
 /**
  * Asks the audio capture filter to route to the selected sound card (currently only used for AAudio and OpenSLES)
  * @param[in] stream The AudioStream object
  * @param[in] sndcard_capture The wanted audio input soundcard
+ * @return '0' if MS_AUDIO_CAPTURE_SET_INTERNAL_ID has been successfully called, '-1' if not
  */
-MS2_PUBLIC void audio_stream_set_input_ms_snd_card(AudioStream *stream, MSSndCard * sndcard_capture);
+MS2_PUBLIC int audio_stream_set_input_ms_snd_card(AudioStream *stream, MSSndCard * sndcard_capture);
 
 /**
  * Asks the audio playback filter to route to the selected sound card (currently only used for AAudio and OpenSLES)
  * @param[in] stream The AudioStream object
  * @param[in] sndcard_playback The wanted audio output soundcard
+ * @return '0' if MS_AUDIO_PLAYBACK_SET_INTERNAL_ID has been successfully called, '-1' if not
  */
-MS2_PUBLIC void audio_stream_set_output_ms_snd_card(AudioStream *stream, MSSndCard * sndcard_playback);
+MS2_PUBLIC int audio_stream_set_output_ms_snd_card(AudioStream *stream, MSSndCard * sndcard_playback);
 
 /**
  * Retrieve the current sound card from the audio capture filter (currently only used for AAudio and OpenSLES)
